/***********************************************************
 *
 *   Program:  PaellaM07      Version: 1.00
 *
 *   Author:  S. Swift					   Date: Sep 9, 2014
 *
 *   This application configures the STM32F072 on a PaellaM07 Sensor Node as a CDC USB device
 *   using the CooCox V1.7.8 IDE and STMicroelectronics STM32F0x2_USB-FS-Device_Lib V1.0.0.
 *
 *   This software can be used without restriction as the basis for other applications
 *   and products.  The STM32F072 is a low cost ($2.50) ARM M0 processor running at 48MHz
 *   with 128K of flash, 16K SRAM, A/D, D/A, SPI, USARTs, I2C, HDMI, etc in 48, 64 and 100
 *   pin QFP and QFN package types.  A good, low cost, processor.  We are building the F072
 *   into several new products.
 *
 *   The application configures the USB peripheral as a CDC device, USART3 to 38,400,N,8,1,
 *   TIM3 as 1msec time delay clock, sets up a few other timers and implements a printf,scanf
 *   for USB CDC and printf3 for USART3.  USART3 is interrupt driven.  The RF24L01 code was
 *   ported but not configured or tested with SPI - TBD.
 *
 *   The Wireless connection is managed as a Master to allow communication with other Nano's and
 *   Picos.  An enumeration function determines what devices are in the network and returns identification\
 *   data.
 *
 *   The PaellaM07 has two I2C ports muxed out to five separate I2C ports through a 1:4 mux selector
 *   designed to hard connect with Femto Sensor Nodes such as the 6DOF FemtoB.
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License. Use at your own risk.
 *
 *   Copyright 2014,  SOC Robotics, Inc.
 ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT 2013 STMicroelectronics</center></h2>
  *
  * Licensed under MCD-ST Liberty SW License Agreement V2, (the "License");
  * You may not use this file except in compliance with the License.
  * You may obtain a copy of the License at:
  *
  *        http://www.st.com/software_license_agreement_liberty_v2
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
 */
/* Includes ------------------------------------------------------------------*/
#include "stm32f0xx.h"
#include "stm32f0xx_conf.h"
#include "stm32072b_eval.h"

#include "usbd_cdc_core.h"
#include "usbd_usr.h"

#include "stdarg.h"
#include "string.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "PaellaM_Defines.h"
#include "PaellaM_Externals.h"
#include "NRF24L01.h"
#include "FXOS8700.h"
#include "MMA8653.h"
#include "SI7013.h"
#include "st7735.h"
#include "colors.h"

USB_CORE_HANDLE  USB_Device_dev ;

/* Private function prototypes -----------------------------------------------*/
void LEDInit(void);
void EXTI0_Config(void);
void TIM_Config(void);
void USARTInit(unsigned long baudrate);

/* Private functions ---------------------------------------------------------*/
/* Uncomment the line below to select your USB clock source */
//#define USE_USB_CLKSOURCE_CRSHSI48   1
//#define USE_USB_CLKSOURCE_PLL        1

#define LED0ON  GPIO_ResetBits(GPIOB,GPIO_Pin_0)
#define LED0OFF  GPIO_SetBits(GPIOB,GPIO_Pin_0)

#define LED1HIGH   GPIOB->BSRR = GPIO_Pin_0
#define LED1LOW	   GPIOB->BRR = GPIO_Pin_0

#define SCK2LOW		GPIO_ResetBits(GPIOB,GPIO_Pin_13)
#define SCK2HIGH	GPIO_SetBits(GPIOB,GPIO_Pin_13)
#define MOSI2LOW	GPIO_ResetBits(GPIOB,GPIO_Pin_15)
#define MOSI2HIGH	GPIO_SetBits(GPIOB,GPIO_Pin_15)

// STM32F042 defines
#define PWMACTIVE  'a'
#define CMDACTIVE  'c'
#define INACTIVE 'i'
#define YES     'y'
#define NO      'n'
#define HIGH	'h'
#define LOW		'l'

TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
TIM_OCInitTypeDef  TIM_OCInitStructure;
TIM_ICInitTypeDef  TIM_ICInitStructure;

FILE *fpout;

// MPU-9150 9 DOF sensor parameters
volatile char mpu9150state=0;
volatile float magposition[6],mpu9150_tempoffset=MPU9150TEMPOFFSET;
extern volatile struct imudata {
	short int ai[3];
	short int gi[3];
	short int mi[3];
	short int temp;
	float magcor[3];
	float af[3];
	float gf[3];
	float mf[3];
	short int tempi;
	short int rolli;
	short int pitchi;
	short int yawi;
	short int pressi;
	float tempf;
	float roll;
	float pitch;
	float yaw;
	float press;
	char mget;
} imu;

// BMP180 Barometer Parameters
volatile unsigned char BMPstate=0,bmp180mode,bmp180tempstate,bmp180pressurestate,bmp180state;
volatile unsigned short int bmpac4,bmpac5,bmpac6,ibmptemperature,ibmppressure;
volatile short int  bmpac1,bmpac2,bmpac3,bmpb1,bmpb2,bmpmb,bmpmc,bmpmd;
volatile int  bmpskipcount;
volatile long b5;
volatile float bmppressure,bmptemperature,pressuremin,pressuremax,bmp180_tempoffset=BMP180TEMPOFFSET;

// FXOS8700 6DOF 3-axis accelerometer, 3-axis magnetometer variables
volatile char fxos8700state=0;
volatile int fingerindex,digitindex;

FXOS_F dip[5];
FXOS_F pip[5];
FXOS_F mcp[5];

volatile FXOSRawData fxosRawData;
volatile char pca9545channel[4] = {PCA9545_CHANNEL_0,PCA9545_CHANNEL_1,PCA9545_CHANNEL_2,PCA9545_CHANNEL_3};
extern FXOS_F FXOS8700CQ_Accel_Mag_Float(I2C_TypeDef* I2Cx,unsigned char address);

// MMA8653 3DOF accelerometer variables
volatile char mma8653state=0;

// SI7013 Humidity sensor variables
volatile char si7013state,si7013sample=SI7013START,si7013sampletarget=SI7013SAMPLEHUMIDITY;
volatile struct sid {
	unsigned int rawtemperature,rawhumidity;
	float temperature,humidity;
	} si7013;

// L3GD20 3DOF gyro sensor variables
volatile char l3gd20state=0;

// ADPS-9007 light sensor variables
volatile char adps9007state=0;

// Display configuration parameters - this is set at init time
volatile char updatedisplay=INACTIVE;
volatile int updaterate=0;

#define RXBUFSIZE 100
char rxbuf[RXBUFSIZE];
int rxheadptr,rxtailptr;

#define     USB_RX_BUFFERSIZE 512

int usbrxheadptr=0, usbrxtailptr=0;
unsigned char usbrxbuffer[USB_RX_BUFFERSIZE];

#define     USART2_TX_BUFSIZE 80
#define     USART2_RX_BUFSIZE 80

int tx2headptr,tx2tailptr,rx2headptr,rx2tailptr;
unsigned char rx2buffer[USART2_RX_BUFSIZE],tx2buffer[USART2_TX_BUFSIZE];

#define     USART3_TX_BUFSIZE 80
#define     USART3_RX_BUFSIZE 80

int tx3headptr,tx3tailptr,rx3headptr,rx3tailptr;
unsigned char rx3buffer[USART3_RX_BUFSIZE],tx3buffer[USART3_TX_BUFSIZE];

// Timer configuration parameters
volatile char flip=0,grnflip=0,redflip=0,ledblink=NO;
uint16_t TimerPeriod = 0;
uint16_t Channel1Pulse=0,Channel2Pulse=0,Channel3Pulse=0,Channel4Pulse=0;
volatile unsigned int mseccountdown1,mseccountdown2,mseccountdown3,bmp180countdown;
volatile unsigned long mseccounter=0;
float ch_dutycycle[6]={0.01,0.01,0.01,0.01,0.01,0.01};

// I2C configuration parameters
__IO uint16_t IC2Value = 0;
__IO uint16_t DutyCycle = 0;
__IO uint32_t Frequency = 0;
#define I2C100KHZ  0x20310A0D
//#define I2C400KHZ  0x00310A0D  // old
#define I2C400KHZ 0x00A00102
//#define I2C400KHZ  0x00E00D38
//#define I2C1000KHZ 0x00310A0D
//#define I2C1000KHZ 0x00A00D26
#define I2C1000KHZ 0x00000000
//#define I2C1000KHZ 0x00400d2c


/* Uncomment the following line to use Timeout_User_Callback I2C_TimeoutUserCallback().
   If This Callback is enabled, it should be implemented by user in main function .
   TS751_TimeoutUserCallback() function is called whenever a timeout condition
   occurs during communication (waiting on an event that doesn't occur, bus
   errors, busy devices ...). */
#define USE_TIMEOUT_USER_CALLBACK

/* Maximum Timeout values for flags and events waiting loops. These timeouts are
   not based on accurate values, they just guarantee that the application will
   not remain stuck if the I2C communication is corrupted.
   You may modify these timeout values depending on CPU frequency and application
   conditions (interrupts routines ...). */
//#define I2C_FLAG_TIMEOUT         ((uint32_t)0x1000)
#define I2C_LONG_TIMEOUT         ((uint32_t)(10 * I2C_FLAG_TIMEOUT))

/**
  * @brief  Timeout user callback function. This function is called when a timeout
  *         condition occurs during communication with TS751. Only protoype
  *         of this function is declared in TS751 driver. Its implementation
  *         may be done into user application. This function may typically stop
  *         current operations and reset the I2C peripheral and TS751.
  *         To enable this function use uncomment the define USE_TIMEOUT_USER_CALLBACK
  *         at the top of this file.
  */
#ifdef USE_TIMEOUT_USER_CALLBACK
 uint8_t I2C_TIMEOUT_UserCallback(void);
#else
 #define I2C_TIMEOUT_UserCallback()  TS751_FAIL
#endif /* USE_TIMEOUT_USER_CALLBACK */

volatile char memsstate=ACTIVESTATE;
volatile unsigned char messageBuf[20],i2ctimedout;
volatile unsigned int i2c_Timeout;

int adc_input[4];
int rf_cnt=0;

/***********************************************
 *
 *  Initialization Functions
 *
 ***********************************************/
/**
  * @brief  Configures LED GPIO.
  */
void LED_Init(void)
{
	GPIO_InitTypeDef  GPIO_InitStructure;

	/* Enable the GPIO_LED Clock */
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);

	/* Configure the GPIO_LED pin */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOB, &GPIO_InitStructure);
}

//  Configures Port A inputs and outputs
//    PA15 - TXD0
//    PA14 - RXD0
//    PA13 - General IO
//    PA12 - D+
//    PA11 - D-
//    PA10 - SDA2
//    PA9  - SCL2
//    PA8  - General IO
//    PA7  - General IO
//    PA6  - General IO
//    PA5  - General IO - DAC2
//    PA4  - General IO - DAC1
//    PA3  - General IO - ADC3
//    PA2  - General IO - ADC2
//    PA1  - General IO - ADC1
//    PA0  - General IO - ADC0
void PortA_Init(void)
{
	GPIO_InitTypeDef  GPIO_InitStructure;

	// Enable Port A outputs
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);

	// Configure the outputs
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_15 | GPIO_Pin_10 | GPIO_Pin_9;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	//GPIO_InitStructure.GPIO_Speed = GPIO_Speed_Level_1 ;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	// Configure the GPIOA input pins
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_14 | GPIO_Pin_13 | GPIO_Pin_8 | GPIO_Pin_7 | GPIO_Pin_6 |
							      GPIO_Pin_5 | GPIO_Pin_4 | GPIO_Pin_3 | GPIO_Pin_2 | GPIO_Pin_1 | GPIO_Pin_0;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
	//GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
}

//  Configures Port B inputs and outputs
//    PB15 - MOSI2
//    PB14 - MISO2
//    PB13 - SCK2
//    PB12 - SS2
//    PB11 - GPIO25  - RF24L01 CSN
//    PB10 - IRQ
//    PB9  - GPIO4
//    PB8  - CE1   - RF24L01 CE
//    PB7  - SDA1
//    PB6  - SCL1
//    PB5  - General IO
//    PB4  - General IO
//    PB3  - General IO
//    PB2  - General IO
//    PB1  - General IO
//    PB0  - LED
void PortB_Init(void)
{
	GPIO_InitTypeDef  GPIO_InitStructure;

	// Enable the PortB Clock
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);

	// Configure the outputs
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12 | GPIO_Pin_11 | GPIO_Pin_8 |
			                      GPIO_Pin_6  | GPIO_Pin_0;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	//GPIO_InitStructure.GPIO_Speed = GPIO_Speed_Level_1 ;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOB, &GPIO_InitStructure);

	// Configure the GPIOB input pins
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10 | GPIO_Pin_9 | GPIO_Pin_5 | GPIO_Pin_4 |
								  GPIO_Pin_3  | GPIO_Pin_2  | GPIO_Pin_1;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
	//GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOB, &GPIO_InitStructure);
}

//  Configures PortC inputs and outputs
//    PC15 - OSC2
//    PC14 - OSC1
//    PC13 -
void PortC_Init(void)
{
	GPIO_InitTypeDef  GPIO_InitStructure;

	// Enable the GPIO_LED Clock
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOC, ENABLE);

	// Configure PortB outputs
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_15 | GPIO_Pin_14 | GPIO_Pin_13;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
	//GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	//GPIO_InitStructure.GPIO_Speed = GPIO_Speed_Level_1 ;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOC, &GPIO_InitStructure);

	// Configure the GPIOC input pins
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11 | GPIO_Pin_5;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
	//GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOC, &GPIO_InitStructure);
}

//  Configure Port F input and outputs
//    PF1  - OSCOUT
//    PF0  - OSCIN
void PortF_Init(void)
{
	GPIO_InitTypeDef  GPIO_InitStructure;

	// Enable the Port F clock
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOF, ENABLE);

	// Configure the 0 1 as outputs
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1 | GPIO_Pin_0;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
	//GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	//GPIO_InitStructure.GPIO_Speed = GPIO_Speed_Level_1 ;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOF, &GPIO_InitStructure);
}

/**
  * @brief  Configures COM port.  Supports buffered interrupt IO
  */
void USART2_Init(unsigned long baudrate)
{
    GPIO_InitTypeDef GPIO_InitStructure;
    USART_InitTypeDef USART_InitStructure;
	NVIC_InitTypeDef NVIC_InitStructure;

    // Initialize interrupt tx rx buffers
    tx2headptr = tx2tailptr = rx2headptr = rx2tailptr = 0;

    /* USARTx configured as follow:
    - BaudRate = 115200 baud
    - Word Length = 8 Bits
    - One Stop Bit
    - No parity
    - Hardware flow control disabled (RTS and CTS signals)
    - Receive and transmit enabled
    */
    USART_InitStructure.USART_BaudRate = baudrate;
    //USART_InitStructure.USART_BaudRate = 115200;
    USART_InitStructure.USART_WordLength = USART_WordLength_8b;
    USART_InitStructure.USART_StopBits = USART_StopBits_1;
    USART_InitStructure.USART_Parity = USART_Parity_No;
    USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
    USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;

    /* Enable GPIO clock */
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);

    /* Enable USART clock */
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2, ENABLE);

    /* Connect PXx to USARTx_Tx */
    GPIO_PinAFConfig(GPIOB, GPIO_PinSource2, GPIO_AF_0);

    /* Connect PXx to USARTx_Rx */
    GPIO_PinAFConfig(GPIOB, GPIO_PinSource3, GPIO_AF_0);

    /* Configure USART Tx, Rx as alternate function push-pull */
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_3 | GPIO_Pin_4;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    /* USART configuration */
    USART_Init(USART2, &USART_InitStructure);

    USART_ITConfig(USART2, USART_IT_RXNE, ENABLE);
    USART_ITConfig(USART2, USART_IT_TXE, ENABLE);

	/* Enable the USART3 global Interrupt */
	NVIC_InitStructure.NVIC_IRQChannel = USART2_IRQn;
//	NVIC_InitStructure.NVIC_IRQChannelPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

    /* Enable USART */
    USART_Cmd(USART2, ENABLE);
}

/**
  * @brief  Configures COM port.  Supports buffered interrupt IO
  */
void USART3_Init(unsigned long baudrate)
{
    GPIO_InitTypeDef GPIO_InitStructure;
    USART_InitTypeDef USART_InitStructure;
	NVIC_InitTypeDef NVIC_InitStructure;

    // Initialize interrupt tx rx buffers
    tx3headptr = tx3tailptr = rx3headptr = rx3tailptr = 0;

    /* USARTx configured as follow:
    - BaudRate = 115200 baud
    - Word Length = 8 Bits
    - One Stop Bit
    - No parity
    - Hardware flow control disabled (RTS and CTS signals)
    - Receive and transmit enabled
    */
    USART_InitStructure.USART_BaudRate = baudrate;
    //USART_InitStructure.USART_BaudRate = 115200;
    USART_InitStructure.USART_WordLength = USART_WordLength_8b;
    USART_InitStructure.USART_StopBits = USART_StopBits_1;
    USART_InitStructure.USART_Parity = USART_Parity_No;
    USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
    USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;

    /* Enable GPIO clock */
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);

    /* Enable USART clock */
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART3, ENABLE);

    /* Connect PXx to USARTx_Tx */
    GPIO_PinAFConfig(GPIOB, GPIO_PinSource10, GPIO_AF_4);

    /* Connect PXx to USARTx_Rx */
    GPIO_PinAFConfig(GPIOB, GPIO_PinSource11, GPIO_AF_4);

    /* Configure USART Tx, Rx as alternate function push-pull */
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10 | GPIO_Pin_11;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_Init(GPIOB, &GPIO_InitStructure);

    /* USART configuration */
    USART_Init(USART3, &USART_InitStructure);

    USART_ITConfig(USART3, USART_IT_RXNE, ENABLE);
    USART_ITConfig(USART3, USART_IT_TXE, ENABLE);

	/* Enable the USART3 global Interrupt */
	NVIC_InitStructure.NVIC_IRQChannel = USART3_4_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

    /* Enable USART */
    USART_Cmd(USART3, ENABLE);
}

/**
  * @brief  Configure the TIM1 Pins for PWM output on PA.
  */
void TIM1_Config(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;

	/* GPIOA Clocks enable */
	RCC_AHBPeriphClockCmd( RCC_AHBPeriph_GPIOA, ENABLE);

	/* GPIOA Configuration: Channel 1, 2 as alternate function push-pull */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8 | GPIO_Pin_11;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP ;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	GPIO_PinAFConfig(GPIOA, GPIO_PinSource8, GPIO_AF_2);
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource11, GPIO_AF_2);
}

// PWM output example - TIM2 can generate four different pwm outputs
void TIM1_ConfigPWM(void)
{
	// Compute the value to be set in ARR register to generate signal frequency at 17.57 Khz
	TimerPeriod = (SystemCoreClock / 17570 ) - 1;
	// Compute CCR1 value to generate a duty cycle at 1% for channel 1
	Channel1Pulse = (uint16_t) (((uint32_t) 1 * (TimerPeriod - 1)) / 1000);
	// Compute CCR2 value to generate a duty cycle at 1%  for channel 2
	Channel2Pulse = (uint16_t) (((uint32_t) 1 * (TimerPeriod - 1)) / 1000);
	// Compute CCR2 value to generate a duty cycle at 1%  for channel 3
	Channel3Pulse = (uint16_t) (((uint32_t) 1 * (TimerPeriod - 1)) / 1000);
	// Compute CCR2 value to generate a duty cycle at 1%  for channel 4
	Channel4Pulse = (uint16_t) (((uint32_t) 1 * (TimerPeriod - 1)) / 1000);

	// TIM1 clock enable
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM1 , ENABLE);

	// Time Base configuration
	TIM_TimeBaseStructure.TIM_Prescaler = 0;
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
	TIM_TimeBaseStructure.TIM_Period = TimerPeriod;
	TIM_TimeBaseStructure.TIM_ClockDivision = 0;
	TIM_TimeBaseStructure.TIM_RepetitionCounter = 0;

	TIM_TimeBaseInit(TIM1, &TIM_TimeBaseStructure);

	// Channel 1, 2, 3 and 4 Configuration in PWM mode
	TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM1;
	TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
	TIM_OCInitStructure.TIM_OutputNState = TIM_OutputNState_Enable;
	TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_Low;
	TIM_OCInitStructure.TIM_OCNPolarity = TIM_OCNPolarity_High;
	TIM_OCInitStructure.TIM_OCIdleState = TIM_OCIdleState_Set;
	TIM_OCInitStructure.TIM_OCNIdleState = TIM_OCIdleState_Reset;

	TIM_OCInitStructure.TIM_Pulse = Channel1Pulse;
	TIM_OC1Init(TIM1, &TIM_OCInitStructure);

	TIM_OCInitStructure.TIM_Pulse = Channel2Pulse;
	TIM_OC2Init(TIM1, &TIM_OCInitStructure);

	TIM_OCInitStructure.TIM_Pulse = Channel3Pulse;
	TIM_OC3Init(TIM1, &TIM_OCInitStructure);

	TIM_OCInitStructure.TIM_Pulse = Channel4Pulse;
	TIM_OC4Init(TIM1, &TIM_OCInitStructure);

	// TIM1 counter enable
	TIM_Cmd(TIM1, ENABLE);

	// TIM1 Main Output Enable
	TIM_CtrlPWMOutputs(TIM1, ENABLE);
}

// PWM timer setup example - input sets pwm output
void TIM2_ConfigIRQ()
{
	/* ---------------------------------------------------------------------------
	TIM2 configuration: PWM Input mode

	In this example TIM2 input clock (TIM2CLK) is set to APB1 clock (PCLK1)
	  TIM2CLK = PCLK1
	  => TIM2CLK = HCLK = SystemCoreClock

	External Signal Frequency = TIM2 counter clock / TIM2_CCR2 in Hz.

	External Signal DutyCycle = (TIM2_CCR1*100)/(TIM2_CCR2) in %.

	--------------------------------------------------------------------------- */

	TIM_ICInitStructure.TIM_Channel = TIM_Channel_2;
	TIM_ICInitStructure.TIM_ICPolarity = TIM_ICPolarity_Rising;
	TIM_ICInitStructure.TIM_ICSelection = TIM_ICSelection_DirectTI;
	TIM_ICInitStructure.TIM_ICPrescaler = TIM_ICPSC_DIV1;
	TIM_ICInitStructure.TIM_ICFilter = 0x0;

	TIM_PWMIConfig(TIM2, &TIM_ICInitStructure);

	/* Select the TIM2 Input Trigger: TI2FP2 */
	TIM_SelectInputTrigger(TIM2, TIM_TS_TI2FP2);

	/* Select the slave Mode: Reset Mode */
	TIM_SelectSlaveMode(TIM2, TIM_SlaveMode_Reset);
	TIM_SelectMasterSlaveMode(TIM2,TIM_MasterSlaveMode_Enable);

	/* TIM enable counter */
	TIM_Cmd(TIM2, ENABLE);

	/* Enable the CC2 Interrupt Request */
	TIM_ITConfig(TIM2, TIM_IT_CC2, ENABLE);
}


// PWM output example - TIM2 can generate four different pwm outputs
// To simulate a typical DC RF transmitter for drone communication
// frequency is about 40Hz with a on time of 4 msec.  So for a resolution
// of 1:65535 a target frequency of 40*1000 or 40,000Hz is required.  The
// pre-scalar should then be 48000000/65535=732 which gives the maximum
// resolution for the small change in the ontime
void TIM2_ConfigPWM(void)
{
	// Compute the value to be set in ARR register to generate signal frequency at 17.57 Khz
	TimerPeriod = (SystemCoreClock / 29 ) - 1;
	// Compute CCR1 value to generate a duty cycle at 5ms for channel 1
	Channel1Pulse = 64000*0.9;
	// Compute CCR2 value to generate a duty cycle at 5ms  for channel 2
	Channel2Pulse = 64000*0.9;
	// Compute CCR2 value to generate a duty cycle at 5ms  for channel 3
	Channel3Pulse = 64000*0.9;
	// Compute CCR2 value to generate a duty cycle at 5ms  for channel 4
	Channel4Pulse = 64000*0.9;

	printf("channel1pulse=%d %x\n\r",Channel1Pulse,Channel1Pulse);

	// TIM1 clock enable
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2 , ENABLE);

	// Time Base configuration
	TIM_TimeBaseStructure.TIM_Prescaler = 2;
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
	TIM_TimeBaseStructure.TIM_Period = 64000;
	TIM_TimeBaseStructure.TIM_ClockDivision = 0;
	TIM_TimeBaseStructure.TIM_RepetitionCounter = 0;

	TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);

	// Channel 1, 2, 3 and 4 Configuration in PWM mode
	TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM1;
	TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
	TIM_OCInitStructure.TIM_OutputNState = TIM_OutputNState_Enable;
	TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_Low;
	TIM_OCInitStructure.TIM_OCNPolarity = TIM_OCNPolarity_High;
	TIM_OCInitStructure.TIM_OCIdleState = TIM_OCIdleState_Set;
	TIM_OCInitStructure.TIM_OCNIdleState = TIM_OCIdleState_Reset;

	TIM_OCInitStructure.TIM_Pulse = Channel1Pulse;
	TIM_OC1Init(TIM2, &TIM_OCInitStructure);

	TIM_OCInitStructure.TIM_Pulse = Channel2Pulse;
	TIM_OC2Init(TIM2, &TIM_OCInitStructure);

	TIM_OCInitStructure.TIM_Pulse = Channel3Pulse;
	TIM_OC3Init(TIM2, &TIM_OCInitStructure);

	TIM_OCInitStructure.TIM_Pulse = Channel4Pulse;
	TIM_OC4Init(TIM2, &TIM_OCInitStructure);

	// TIM1 counter enable
	TIM_Cmd(TIM2, ENABLE);

	// TIM1 Main Output Enable
	TIM_CtrlPWMOutputs(TIM2, ENABLE);
}

/**
  * @brief  Configure the TIM2 IRQ Handler.
  */
void TIM2_Config(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;
	NVIC_InitTypeDef NVIC_InitStructure;

	/* TIM2 clock enable */
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);

	/* GPIOA clock enable */
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);

	/* TIM2 chennel2 configuration : PA.01 */
	GPIO_InitStructure.GPIO_Pin   = GPIO_Pin_3 | GPIO_Pin_2 | GPIO_Pin_1 | GPIO_Pin_0;
	GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_UP ;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	/* Connect TIM pin to AF2 */
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource0, GPIO_AF_2);
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource1, GPIO_AF_2);
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource2, GPIO_AF_2);
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource3, GPIO_AF_2);

	TIM2_ConfigPWM();

	/* Enable the TIM2 global Interrupt */
	//NVIC_InitStructure.NVIC_IRQChannel = TIM2_IRQn;
	//NVIC_InitStructure.NVIC_IRQChannelPriority = 0;
	//NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	//NVIC_Init(&NVIC_InitStructure);
}

/**
  * @brief  Configure the TIM3 IRQ Handler.
  */
void TIM3_Config(void)
{
	NVIC_InitTypeDef NVIC_InitStructure;

	/* TIM3 clock enable */
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE);

	/* Enable the TIM3 global Interrupt */
	NVIC_InitStructure.NVIC_IRQChannel = TIM3_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);
}

// Setup Timer 3 as a 1msec time delay interrupt
void TIM3_Init(void)
{
	uint16_t CCR3_Val,PrescalerValue;

	/* TIM Configuration */
	TIM3_Config();

	/* -----------------------------------------------------------------------
	TIM3 Configuration: Output Compare Timing Mode:

	In this example TIM3 input clock (TIM3CLK) is set to APB1 clock (PCLK1),
	  => TIM3CLK = PCLK1 = SystemCoreClock = 48 MHz

	To get TIM3 counter clock at 6 MHz, the prescaler is computed as follows:
	   Prescaler = (TIM3CLK / TIM3 counter clock) - 1
	   Prescaler = (PCLK1 /6 MHz) - 1

	CC3 update rate = TIM3 counter clock / CCR3_Val = 439.4 Hz
	==> Toggling frequency = 219.7 Hz

	----------------------------------------------------------------------- */

	/* Compute the prescaler value */
	PrescalerValue = (uint16_t) (SystemCoreClock  / 6000000) - 1;
	//PrescalerValue = 0;

	CCR3_Val = 1000;
	mseccountdown1 = mseccountdown2 = mseccountdown3 = 0;

	/* Time base configuration */
	TIM_TimeBaseStructure.TIM_Period = 65535;
	TIM_TimeBaseStructure.TIM_Prescaler = 0;
	TIM_TimeBaseStructure.TIM_ClockDivision = 0;
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;

	TIM_TimeBaseInit(TIM3, &TIM_TimeBaseStructure);

	/* Prescaler configuration */
	TIM_PrescalerConfig(TIM3, PrescalerValue, TIM_PSCReloadMode_Immediate);

	/* Output Compare Timing Mode configuration: Channel1 */
	//TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_Timing;
	//TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;

	/* Output Compare Timing Mode configuration: Channel3 */
	TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
	TIM_OCInitStructure.TIM_Pulse = CCR3_Val;

	TIM_OC3Init(TIM3, &TIM_OCInitStructure);

	TIM_OC3PreloadConfig(TIM3, TIM_OCPreload_Disable);

	/* TIM Interrupts enable */
	TIM_ITConfig(TIM3, TIM_IT_CC3, ENABLE);

	/* TIM3 enable counter */
	TIM_Cmd(TIM3, ENABLE);
}

// Setup usec delay timer
void TIM15_Init(void)
{
	uint16_t CCR3_Val,PrescalerValue;

	/* TIM15 clock enable */
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM15, ENABLE);

	/* Compute the prescaler value */
	PrescalerValue = (uint16_t) (SystemCoreClock  / 1000000) - 1;
	//PrescalerValue = 0;

	CCR3_Val = 0;

	/* Time base configuration */
	TIM_TimeBaseStructure.TIM_Period = 65535;
	TIM_TimeBaseStructure.TIM_Prescaler = 0;
	TIM_TimeBaseStructure.TIM_ClockDivision = 0;
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;

	TIM_TimeBaseInit(TIM15, &TIM_TimeBaseStructure);

	/* Prescaler configuration */
	TIM_PrescalerConfig(TIM15, PrescalerValue, TIM_PSCReloadMode_Immediate);

	/* Output Compare Timing Mode configuration: Channel3 */
	//TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
	//TIM_OCInitStructure.TIM_Pulse = CCR3_Val;

	//TIM_OC3Init(TIM15, &TIM_OCInitStructure);

	//TIM_OC1PreloadConfig(TIM15, TIM_OCPreload_Disable);

	/* TIM Interrupts enable */
	//TIM_ITConfig(TIM15, TIM_IT_CC1, ENABLE);

	/* TIM3 enable counter */
	//TIM_Cmd(TIM15, ENABLE);
}

// I2C1 is attached to the optional OLED connector, MPU-9150 chip
// and the Raspberry Pi I2C port.  This function is enabled by default
// by PaellaM07 so the Raspberry Pi should not enable the I2C port.
// Default I2C speed is 400KHz but can operate up to 1000KHz.
void I2C1_Init(unsigned long speed)
{
	I2C_InitTypeDef  I2C1_InitStructure;
	GPIO_InitTypeDef GPIO_InitStructure;
    NVIC_InitTypeDef NVIC_InitStructure;

	I2C_DeInit(I2C1);

	// I2C Periph clock enable
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_I2C1, ENABLE);
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);

    // Configure the I2C clock source. The clock is derived from the HSI
    RCC_I2CCLKConfig(RCC_I2C1CLK_HSI);

    GPIO_PinAFConfig(GPIOB, GPIO_PinSource6, GPIO_AF_1);
    GPIO_PinAFConfig(GPIOB, GPIO_PinSource7, GPIO_AF_1);

    // Configure pins: SCL and SDA ------------------
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;
    GPIO_Init(GPIOB, &GPIO_InitStructure);

    // Init I2C to defaults
    I2C_StructInit(&I2C1_InitStructure);

    I2C1_InitStructure.I2C_AnalogFilter = I2C_AnalogFilter_Enable;
    I2C1_InitStructure.I2C_DigitalFilter = 0x00;
    I2C1_InitStructure.I2C_OwnAddress1 = 0x00;
    I2C1_InitStructure.I2C_Mode = I2C_Mode_I2C;
    I2C1_InitStructure.I2C_Ack = I2C_Ack_Enable;

    /* I2C TIMING Register define when I2C clock source is SYSCLK */
    /* I2C TIMING is calculated in case of the I2C Clock source is the SYSCLK = 72 MHz */
    /* When using SFOX8700 set TIMING to 0x00C4092A to reach 1 MHz speed (Rise time = 26ns, Fall time = 2ns) */
    /* When using slow devices set TIMING to 0xC062121F to reach 100 KHz speed (Rise time = 640ns, Fall time = 20ns) */

    //I2C_InitStructure.I2C_Timing = 0x0x00C4092A;  // 1MHz
    //I2C1_InitStructure.I2C_Timing = 0xC062121F;  // 100KHz
    //I2C1_InitStructure.I2C_Timing = 0x20310A0D;  // 400KHz
    //I2C1_InitStructure.I2C_Timing = 0x50330309; // 400KHz - other source
    I2C1_InitStructure.I2C_Timing = speed;
    I2C1_InitStructure.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;

    I2C_Init(I2C1, &I2C1_InitStructure);

	// Configure the I2C interrupt priority
	//NVIC_InitStructure.NVIC_IRQChannel = I2C1_EV_IRQn;
	//NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
	//NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	//NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	//NVIC_Init(&NVIC_InitStructure);

	// Enable I2C
	I2C_Cmd(I2C1, ENABLE);

	// Enable Interrupts
	//I2C_ITConfig(I2C1, I2C_IT_ADDRI | I2C_IT_RXI, ENABLE);
}

void I2C2_Init(unsigned long speed)
{
	I2C_InitTypeDef  I2C2_InitStructure;
	GPIO_InitTypeDef GPIO_InitStructure;
    NVIC_InitTypeDef NVIC_InitStructure;

	I2C_DeInit(I2C2);

	// I2C Periph clock enable
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_I2C2, ENABLE);
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);

    // Configure the I2C clock source. The clock is derived from the HSI
    RCC_I2CCLKConfig(RCC_I2C1CLK_HSI);

    GPIO_PinAFConfig(GPIOA, GPIO_PinSource9, GPIO_AF_4);
    GPIO_PinAFConfig(GPIOA, GPIO_PinSource10, GPIO_AF_4);

    //Configure pins: SCL and SDA ------------------
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9 | GPIO_Pin_10;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    I2C2_InitStructure.I2C_AnalogFilter = I2C_AnalogFilter_Enable;
    I2C2_InitStructure.I2C_DigitalFilter = 0x00;
    I2C2_InitStructure.I2C_OwnAddress1 = 0x00;
    I2C2_InitStructure.I2C_Mode = I2C_Mode_I2C;
    I2C2_InitStructure.I2C_Ack = I2C_Ack_Enable;

    /* I2C TIMING Register define when I2C clock source is SYSCLK */
    /* I2C TIMING is calculated in case of the I2C Clock source is the SYSCLK = 72 MHz */
    /* When using SFOX8700 set TIMING to 0x00C4092A to reach 1 MHz speed (Rise time = 26ns, Fall time = 2ns) */
    /* When using slow devices set TIMING to 0xC062121F to reach 100 KHz speed (Rise time = 640ns, Fall time = 20ns) */

    //I2C_InitStructure.I2C_Timing = 0x0x00C4092A;  // 1MHz
    //I2C1_InitStructure.I2C_Timing = 0xC062121F;  // 100KHz
    //I2C1_InitStructure.I2C_Timing = 0x20310A0D;  // 400KHz
    //I2C1_InitStructure.I2C_Timing = 0x50330309; // 400KHz - other source
    I2C2_InitStructure.I2C_Timing = speed;
    I2C2_InitStructure.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;

    I2C_Init(I2C2, &I2C2_InitStructure);

    I2C_Cmd(I2C2, ENABLE);
}

// SPI1 is available on PA5,6,7 or PB3,4,5
// Th3 function defaults to PB3,4,5 - use PB2 as CS but is
// called by default by the PaellaM07_Init function
void SPI1_Init(void)
{
    GPIO_InitTypeDef GPIO_InitStructure;
    SPI_InitTypeDef SPI_InitStructure;

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_SPI1, ENABLE);
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB | RCC_AHBPeriph_GPIOE, ENABLE);

    // Configure IO for PB3,4,5
    GPIO_PinAFConfig(GPIOB, GPIO_PinSource3, GPIO_AF_0); // SPI SCK
    GPIO_PinAFConfig(GPIOB, GPIO_PinSource4, GPIO_AF_0); // SPI MISO
    GPIO_PinAFConfig(GPIOB, GPIO_PinSource5, GPIO_AF_0); // SPI MOSI

    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;//GPIO_PuPd_DOWN;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_3 | GPIO_Pin_4 | GPIO_Pin_5;
    GPIO_Init(GPIOB, &GPIO_InitStructure);

    SPI_I2S_DeInit(SPI1);
    SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
    SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
    SPI_InitStructure.SPI_CPOL = SPI_CPOL_Low;
    SPI_InitStructure.SPI_CPHA = SPI_CPHA_1Edge;
    SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;
    SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_8;
    SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
    SPI_InitStructure.SPI_CRCPolynomial = 7;
    SPI_InitStructure.SPI_Mode = SPI_Mode_Master;
    SPI_Init(SPI1, &SPI_InitStructure);

    SPI_RxFIFOThresholdConfig(SPI1, SPI_RxFIFOThreshold_QF);

    SPI_Cmd(SPI1, ENABLE);

    // Ensure RXNE is initially clear
    SPI_ReceiveData8(SPI1);
}

// SPI2 is on PortB and is attached the RF24L01 port pins and Raspberry Pi
// expansion connector.  Control of the RF24L01 is shared by lines on the
// Raspberry Pi and Paella - only the STM32F072 or Raspberry Pi should
// communicate with the RF24L01 - not both at the same time.
//   PB13 - SCK2
//   PB14 - MISO2
//   PB15 - MOSI2
//   RF24L01 CE - PB11 which is also attached to Raspberry Pi CE1
//   RF24L01 CSN - PB8 which is also attached to Raspberry Pi GPIO25
void SPI2_Init(void)
{
    GPIO_InitTypeDef GPIO_InitStructure;
    SPI_InitTypeDef SPI_InitStructure;

    if(1) {
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_SPI2, ENABLE);
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);

    // Configure IO for nRF24L01 SPI SCK, MISO, MOSI
    GPIO_PinAFConfig(GPIOB, GPIO_PinSource13, GPIO_AF_0); // SPI SCK
    GPIO_PinAFConfig(GPIOB, GPIO_PinSource14, GPIO_AF_0); // SPI MISO
    GPIO_PinAFConfig(GPIOB, GPIO_PinSource15, GPIO_AF_0 ); // SPI MOSI

    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;//GPIO_PuPd_DOWN;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15;
    GPIO_Init(GPIOB, &GPIO_InitStructure);

	// GPIO SPI2 CS - CE-PB8, CSN-PB11
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8 | GPIO_Pin_11;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	// OType, Speed and PuPd are the same
	GPIO_Init(GPIOB, &GPIO_InitStructure);

    SPI_I2S_DeInit(SPI2);
    SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
    SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
    SPI_InitStructure.SPI_CPOL = SPI_CPOL_Low;
    SPI_InitStructure.SPI_CPHA = SPI_CPHA_1Edge;
    SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;
    //SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_8; // 4mbit rate
    SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_16;  // 2mbit rate
    SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
    SPI_InitStructure.SPI_CRCPolynomial = 7;
    SPI_InitStructure.SPI_Mode = SPI_Mode_Master;
    SPI_Init(SPI2, &SPI_InitStructure);

    SPI_RxFIFOThresholdConfig(SPI2, SPI_RxFIFOThreshold_QF);

    //printf("\n\rSPI_BaudRatePrescaler_8=%d\n\r",SPI_BaudRatePrescaler_8);

    SPI_Cmd(SPI2, ENABLE);

    // Ensure RXNE is initially clear
    SPI_ReceiveData8(SPI2);
	}
	// Bit bang SPI function
	else {
	    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);

		// GPIO SPI2 CS - CE-PB11, CSN-PB12, SCK2-PB13, MOSI2-PB15
		GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_15;
		GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
		// OType, Speed and PuPd are the same
		GPIO_Init(GPIOB, &GPIO_InitStructure);

		// GPIO SPI2 Input - MISO2-PB14
		GPIO_InitStructure.GPIO_Pin = GPIO_Pin_14;
		GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
		// OType, Speed and PuPd are the same
		GPIO_Init(GPIOB, &GPIO_InitStructure);

		MOSI2HIGH;
		SCK2LOW;
		}
}

uint8_t SPI1_transfer_byte(uint8_t byte)
{
	while (SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_TXE) == RESET);
	// Send a Byte through the SPI peripheral
	SPI_SendData8(SPI1, byte);
	while (SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_RXNE) == RESET);
	return (uint8_t)SPI_ReceiveData8(SPI1);
}

// Bit bang version - pretty slow
uint8_t SPI2_transfer_byte_old(uint8_t outbyte)
{
	unsigned char inbyte,mask=0x80;
	int i;

	inbyte = 0;
	for(i=0; i<8; i++) {
		if(outbyte&mask) MOSI2HIGH;
		else MOSI2LOW;
		delay_us(1);
		SCK2HIGH;
		if(GPIO_ReadInputDataBit(GPIOB,GPIO_Pin_14)) inbyte |= mask;
		delay_us(1);
		SCK2LOW;
		//delay_us(2);
		mask = mask >> 1;
		}
	MOSI2HIGH;
	return inbyte;
}

uint8_t SPI2_transfer_byte(uint8_t byte)
{

	while (SPI_I2S_GetFlagStatus(SPI2, SPI_I2S_FLAG_TXE) == RESET);

	// Send a Byte through the SPI peripheral
	SPI_SendData8(SPI2, byte);
	while (SPI_I2S_GetFlagStatus(SPI2, SPI_I2S_FLAG_RXNE) == RESET);
	return (uint8_t)SPI_ReceiveData8(SPI2);
}

// Assume PA0-3 are ADC inputs
void Init_ADC()
{
	int calibration_value;

	ADC_InitTypeDef ADC_init_structure; //Structure for ADC configuration
	GPIO_InitTypeDef GPIO_initStructre; //Structure for analog input pin
	//ADC_CommonInitTypeDef  ADC_CommonInitStructure;

	//RCC_ADCCLKConfig(RCC_ADC12PLLCLK_Div2);

	//Clock configuration
	RCC_AHBPeriphClockCmd(RCC_APB2Periph_ADC1,ENABLE); //The ADC12 is connected the APB peripheral bus thus we will use its clock source
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA,ENABLE); //Clock for the ADC port!! Do not forget about this one ;)

	//Analog pin configuration
	GPIO_initStructre.GPIO_Pin = GPIO_Pin_3|GPIO_Pin_2|GPIO_Pin_1|GPIO_Pin_0;
	GPIO_initStructre.GPIO_Mode = GPIO_Mode_AN; //The PA0-3 pin is configured in analog mode
	GPIO_initStructre.GPIO_PuPd = GPIO_PuPd_NOPULL; //We don't need any pull up or pull down
	GPIO_Init(GPIOA,&GPIO_initStructre); //Affecting the port with the initialization structure configuration

	//ADC_DeInit(ADC1);
	ADC_StructInit(&ADC_init_structure);

	//ADC_VoltageRegulatorCmd(ADC1, ENABLE);
	delay_us(10);

	//ADC_SelectCalibrationMode(ADC1, ADC_CalibrationMode_Single);
	//ADC_StartCalibration(ADC1);

	//while(ADC_GetCalibrationStatus(ADC1) != RESET );
	//calibration_value = ADC_GetCalibrationValue(ADC1);


	/* ADC configuration */
	//ADC_CommonInitStructure.ADC_Mode = ADC_Mode_Independent;
	//ADC_CommonInitStructure.ADC_Clock = ADC_Clock_AsynClkMode;
	//ADC_CommonInitStructure.ADC_DMAAccessMode = ADC_DMAAccessMode_Disabled; // Not Multi Mode
	//ADC_CommonInitStructure.ADC_DMAMode = ADC_DMAMode_Circular;
	//ADC_CommonInitStructure.ADC_TwoSamplingDelay = 0;
	//ADC_CommonInit(ADC1,&ADC_CommonInitStructure);

	//ADC structure configuration
	ADC_init_structure.ADC_DataAlign = ADC_DataAlign_Right; //data converted will be shifted to right
	ADC_init_structure.ADC_Resolution = ADC_Resolution_12b; //Input voltage is converted into a 12bit number giving a maximum value of 4096
	ADC_init_structure.ADC_ContinuousConvMode = DISABLE; //the conversion is continuous, the input data is converted more than once
	ADC_init_structure.ADC_ExternalTrigConv = ADC_ExternalTrigConvEdge_None; // conversion is synchronous with TIM1 and CC1 (actually I'm not sure about this one :/)
	ADC_init_structure.ADC_ExternalTrigConvEdge = ADC_ExternalTrigConvEdge_None; //no trigger for conversion
	ADC_init_structure.ADC_ScanDirection = ADC_ScanDirection_Upward; //The scan is configured
	ADC_Init(ADC1,&ADC_init_structure); //Initialize ADC with the previous configuration


	//Enable ADC conversion
	ADC_Cmd(ADC1,ENABLE);
}

// Get ADC data
int ADC_Convert(unsigned char channel)
{
	//channel = ADC_Channel_10;
	//Select the channel to be read from
	ADC_RegularChannelConfig(ADC1,channel,1,ADC_SampleTime_1_5Cycles);
	//ADC_RegularChannelConfig(ADC1,channel,1,ADC_SampleTime_144Cycles);

	//Start the conversion
	ADC_StartConversion(ADC1);
	//printf("Stat=%x ",ADC1->ISR);
	//delay_us(300);

	//printf("ADC_Status=%x ",ADC1->ISR);

	while(!ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC)); //Processing the conversion

	return ADC_GetConversionValue(ADC1); //Return the converted data
}

void Sample_ADC_Channels(void)
{
	adc_input[0] = ADC_Convert(ADC_Channel_1);
	adc_input[1] = ADC_Convert(ADC_Channel_2);
	adc_input[2] = ADC_Convert(ADC_Channel_3);
	adc_input[3] = ADC_Convert(ADC_Channel_4);
}



// DAC1 is on PA5, DAC2 is on PA4
void Init_DAC(void)
{
	DAC_InitTypeDef DAC_InitStructure;
	GPIO_InitTypeDef GPIO_InitStructure_DAC;

	/* GPIOA Periph clock enable */
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1 | RCC_AHBPeriph_GPIOA, ENABLE);

	/* Enable DAC clock */
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_DAC, ENABLE);

	/* DAC channel 1 & 2 (DAC1_2 = PA.5)configuration */
	GPIO_InitStructure_DAC.GPIO_Pin = GPIO_Pin_5;
	GPIO_InitStructure_DAC.GPIO_Mode = GPIO_Mode_AN;
	GPIO_InitStructure_DAC.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init(GPIOA, &GPIO_InitStructure_DAC);

	/* DAC channel1 & DAC_Channel_2 Configuration */
	DAC_InitStructure.DAC_Trigger = DAC_Trigger_None;
	DAC_InitStructure.DAC_WaveGeneration = DAC_WaveGeneration_None;
	DAC_InitStructure.DAC_OutputBuffer = DAC_OutputBuffer_Enable;
	//DAC_Init(DAC_Channel_2, &DAC_InitStructure);
	DAC_Init(DAC_Channel_1, &DAC_InitStructure);

	/* Enable DAC Channel1 & Channel2: Once the DAC is enabled, PA.05 is
	 automatically connected to the DAC converter. */
	//DAC_Cmd(DAC_Channel_2, ENABLE);
	DAC_Cmd(DAC_Channel_1, ENABLE);
}

void Set_DAC_Ch1_Voltage(int voltage)
{
	DAC_SetChannel1Data(DAC_Align_12b_R, voltage);
}


/***********************************************
 *
 *  ISRs
 *
 ***********************************************/
/**
  * @brief  This function handles TIM2 global interrupt request.
  */
void TIM2_IRQHandler(void)
{
	/* Clear TIM2 Capture compare interrupt pending bit */
	//TIM_ClearITPendingBit(TIM2, TIM_IT_CC2);
	TIM2->SR = (uint16_t)~TIM_IT_CC2;

	/* Get the Input Capture value */
	//IC2Value = TIM_GetCapture2(TIM2);
	IC2Value = TIM2->CCR2;
}

// 1 msec interrupt handler for delay countdown
void TIM3_IRQHandler(void)
{
    if (TIM_GetITStatus(TIM3, TIM_IT_CC3) != RESET)  {
		TIM_ClearITPendingBit(TIM3, TIM_IT_CC3);
		//LED2HIGH;
		if(mseccountdown1) mseccountdown1--;
		if(mseccountdown2) mseccountdown2--;
		if(mseccountdown3) mseccountdown3--;
		if(bmp180countdown) bmp180countdown--;
		mseccounter++;
		TIM_SetCounter(TIM3, 0);
		//LED2LOW;
    	}
}

// Interrupt handler for USART3  TX and RX buffer size set by USART3_TX_BUFSIZE and USART3_RX_BUFSIZE
void USART3_4_IRQHandler(void)
{
	// Check receive interrupt
    if (USART_GetITStatus(USART3, USART_IT_RXNE) != RESET)  {
    	USART_ClearITPendingBit(USART3, USART_IT_RXNE);
    	//LED1LOW;
		rx3buffer[rx3headptr++] = (uint8_t)(USART3->RDR & (uint16_t)0x00FF);
		if(rx3headptr==USART3_RX_BUFSIZE) rx3headptr = 0;
		if(rx3headptr==rx3tailptr) {
			rx3tailptr++;
			if(rx3tailptr==USART3_RX_BUFSIZE) rx3tailptr = 0;
			}
    	}
    // Check transmit interrupt
    if (USART_GetITStatus(USART3, USART_IT_TXE) != RESET)  {
    	USART_ClearITPendingBit(USART3, USART_IT_TXE);
    	//LED2LOW;
		USART3->TDR = tx3buffer[tx3tailptr++];
		if(tx3tailptr==USART3_TX_BUFSIZE) tx3tailptr = 0;
		// Turn off interrupt is not more data to transmit
		if(tx3tailptr==tx3headptr)  USART_ITConfig(USART3, USART_IT_TXE, DISABLE);
    	}
}

/***********************************************
 *  I2C Functions
 ***********************************************/
#define INIT_TIMEOUT i2c_Timeout = I2C_LONG_TIMEOUT
#define CHECK_TIMEOUT if((i2c_Timeout--) == 0) return I2C_TIMEOUT_UserCallback()

uint8_t I2C_TIMEOUT_UserCallback(void)
{
	i2ctimedout = YES;
	return i2ctimedout;
}

//  Write to the specified number of bytes to the I2C device
uint8_t I2C_WriteBytes(I2C_TypeDef* I2Cx,uint8_t twiaddress,uint8_t nbytes)
{
	int i=0;
	i2ctimedout = NO;

	// Test on BUSY Flag
	INIT_TIMEOUT;
	while(I2C_GetFlagStatus(I2Cx, I2C_ISR_BUSY) != RESET) {CHECK_TIMEOUT;}

	// Configure slave address, nbytes, reload, end mode and start or stop generation
	I2C_TransferHandling(I2Cx, twiaddress<<1, nbytes, I2C_AutoEnd_Mode, I2C_Generate_Start_Write);

	while(nbytes--) {
		// Wait until TXIS flag is set
		INIT_TIMEOUT;
		while(I2C_GetFlagStatus(I2Cx, I2C_ISR_TXIS) == RESET)  {CHECK_TIMEOUT;}

		// Send Register address
		I2C_SendData(I2Cx, (uint8_t)messageBuf[i++]);
		}

	// Wait until STOPF flag is set
	INIT_TIMEOUT;
	while(I2C_GetFlagStatus(I2Cx, I2C_ISR_STOPF) == RESET) {CHECK_TIMEOUT;}

	// Clear STOPF flag
	I2C_ClearFlag(I2Cx, I2C_ICR_STOPCF);

	return I2C_OK;
}

// Read the specified number of bytes from the I2C device.
// I2Cx - Select I2C1 or I2C2
// twiaddress - I2C address (must shift left one bit
// nbytes - Number of bytes to write
uint8_t I2C_ReadBytes(I2C_TypeDef* I2Cx,uint8_t twiaddress,uint8_t nbytes)
{
	int i=0;
	i2ctimedout = NO;

	// Test on BUSY Flag
	INIT_TIMEOUT;
	while(I2C_GetFlagStatus(I2Cx, I2C_ISR_BUSY) != RESET) {CHECK_TIMEOUT;}

	// Configure slave address, nbytes, reload, end mode and start or stop generation
	I2C_TransferHandling(I2Cx, twiaddress<<1, nbytes, I2C_AutoEnd_Mode, I2C_Generate_Start_Read);

	while(nbytes--) {
		// Wait until RXNE flag is set
		INIT_TIMEOUT;
		while(I2C_GetFlagStatus(I2Cx, I2C_ISR_RXNE) == RESET) {CHECK_TIMEOUT;}

		// Read data from RXDR
		messageBuf[i++]= I2C_ReceiveData(I2Cx);
		}

	// Wait until STOPF flag is set
	INIT_TIMEOUT;
	while(I2C_GetFlagStatus(I2Cx, I2C_ISR_STOPF) == RESET) {CHECK_TIMEOUT;}

	// Clear STOPF flag
	I2C_ClearFlag(I2Cx, I2C_ICR_STOPCF);

	// return a Reg value
	return messageBuf[0];
}

// Read the specified register from the I2C device using a repeated start - combined
// I2Cx - Select I2C1 or I2C2
// twiaddress - I2C address (must shift left one bit
// RegId - specifies the device register to be read.
// nbytes - number of bytes to read
uint8_t I2C_WriteReadBytes(I2C_TypeDef* I2Cx,uint8_t twiaddress,uint8_t RegId,uint8_t nbytes)
{
	int i=0;
	i2ctimedout = NO;

	// Test on BUSY Flag
	INIT_TIMEOUT;
	while(I2C_GetFlagStatus(I2Cx, I2C_ISR_BUSY) != RESET) {CHECK_TIMEOUT;}

	// Configure slave address, nbytes, reload, end mode and start or stop generation
	I2C_TransferHandling(I2Cx, twiaddress<<1, 1, I2C_SoftEnd_Mode, I2C_Generate_Start_Write);

	// Wait until TXIS flag is set
	INIT_TIMEOUT;
	while(I2C_GetFlagStatus(I2Cx, I2C_ISR_TXIS) == RESET) {CHECK_TIMEOUT;}

	// Send Register address
	I2C_SendData(I2Cx, (uint8_t)RegId);

	// NEED TO GENERATE RESTART CONDITION HERE
	//I2C_GenerateSTART(I2C1, ENABLE);
	//while(I2C_GetFlagStatus(I2C1,I2C_FLAG_BUSY));

	// Wait until TC flag is set
	INIT_TIMEOUT;
	while(I2C_GetFlagStatus(I2Cx, I2C_ISR_TC) == RESET) {CHECK_TIMEOUT;}

	// Configure slave address, nbytes, reload, end mode and start or stop generation
	I2C_TransferHandling(I2Cx, twiaddress<<1, nbytes, I2C_AutoEnd_Mode, I2C_Generate_Start_Read);
	//I2C_GenerateSTART(I2C1, ENABLE);

	while(nbytes--) {
		// Wait until RXNE flag is set
		INIT_TIMEOUT;
		while(I2C_GetFlagStatus(I2Cx, I2C_ISR_RXNE) == RESET) {CHECK_TIMEOUT;}

		// Read data from RXDR
		messageBuf[i++]= I2C_ReceiveData(I2Cx);
		}

	// Wait until STOPF flag is set
	INIT_TIMEOUT;
	while(I2C_GetFlagStatus(I2Cx, I2C_ISR_STOPF) == RESET) {CHECK_TIMEOUT;}

	// Clear STOPF flag
	I2C_ClearFlag(I2Cx, I2C_ICR_STOPCF);

	// return a Reg value
	return messageBuf[0];
}

/***********************************************
 *
 *  Serial IO Functions for USB and USART3
 *
 ***********************************************/
int GetCharnw3(void)
{
	int intchar;

	// Uncomment this call if rx buffer is experiencing an overrun condition
	//if(USART_GetFlagStatus(USART3, USART_FLAG_ORE) == SET) {
	//	USART_ClearFlag(USART3, USART_FLAG_ORE);
	//	}

	if(rx3headptr!=rx3tailptr) {
		intchar = rx3buffer[rx3tailptr++];
		if(rx3tailptr==USART3_RX_BUFSIZE) rx3tailptr = 0;
		return intchar;
		}
	else return -1;
}

unsigned char GetChar3(void)
{
	int intchar;

	while((intchar=GetCharnw3())==-1) ;
	return (unsigned char) intchar;
}

void PutChar3(unsigned char outchar)
{
	if(tx3headptr==tx3tailptr) {
		tx3buffer[tx3headptr++] = outchar;
		if(tx3headptr==USART3_TX_BUFSIZE) tx3headptr = 0;
		USART_ITConfig(USART3, USART_IT_TXE, ENABLE);
		}
	else {
		tx3buffer[tx3headptr++] = outchar;
		if(tx3headptr==USART3_TX_BUFSIZE) tx3headptr = 0;
		if(tx3tailptr==tx3headptr) {
			tx3tailptr++;
			if(tx3tailptr==USART3_TX_BUFSIZE) tx3tailptr = 0;
			}
		}
}

void Puts3(unsigned char *outstrg)
{
	int i;

	for(i=0; i<strlen(outstrg); i++) PutChar3(outstrg[i]);
}

// Formated output for USART3
int printf3(const char *fmt, ...)
{
	va_list args;
	int i,j;
	char printbuffer[150];

	va_start (args, fmt);
	i = vsnprintf (printbuffer, 150, fmt, args);
	va_end (args);

	// Print the string
	j = 0;
	while(printbuffer[j]!=0) PutChar3(printbuffer[j++]);

	return i;
}

int scanf3(const char *fmt, ...)
{
	char inchar;
	va_list args;
	int i,ptr;
	char scanbuffer[120];

	ptr = 0;
	while((inchar=GetChar3())!=CRR) {
        scanbuffer[ptr++] = inchar;
		//putchar(inchar);
		}
	scanbuffer[ptr] = 0;

	va_start (args, fmt);
	i = vsscanf(scanbuffer, fmt, args);
	va_end (args);

	return i;
}

// Buffers for USB serial IO
// Hack into Serial to USB bridge code - this could be cleaned up and streamlined
extern uint8_t  APP_Rx_Buffer []; /* Write CDC received data in this buffer.
                                     These data will be sent over USB IN endpoint
                                     in the CDC core functions. */
extern uint32_t APP_Rx_ptr_in;    /* Increment this pointer or roll it back to
                                     start address when writing received data
                                     in the buffer APP_Rx_Buffer. */

/**
  * @brief  VCP_DataTx
  *         CDC received data to be send over USB IN endpoint are managed in
  *         this function.
  * @param  Buf: Buffer of data to be sent
  * @param  Len: Number of data to be sent (in bytes)
  * @retval Result of the operation: USBD_OK if all operations are OK else VCP_FAIL
  */
static uint16_t VCP_DataTx2(uint8_t* Buf, uint32_t Len)
{
	int i;

    for(i=0; i<Len; i++) {
    	APP_Rx_Buffer[APP_Rx_ptr_in++] = Buf[i];
    	//PutChar(Buf[i]);
    	if(APP_Rx_ptr_in == APP_RX_DATA_SIZE)  APP_Rx_ptr_in = 0;
    	}
  return USBD_OK;
}

// USB putchar
void PutChar(unsigned char outchar)
{
	unsigned char outbuf[2];

	outbuf[0] = outchar;
	VCP_DataTx2(outbuf,1);
}

// USB no wait getchar
int GetCharnw(void)
{
	int intchar;

	if(usbrxheadptr!=usbrxtailptr) {
		intchar = usbrxbuffer[usbrxtailptr++];
		if(usbrxtailptr==USB_RX_BUFFERSIZE) usbrxtailptr = 0;
		return intchar;
		}
	else return -1;
}

// USB wait getchar
unsigned char GetChar(void)
{
	int intchar;

	while((intchar=GetCharnw())==-1) ;
	return (unsigned int) intchar;
}

// Formated USB serial output
int printf(const char *fmt, ...)
{
	va_list args;
	int i;
	char printbuffer[150];

	va_start (args, fmt);
	i = vsnprintf (printbuffer, 150, fmt, args);
	va_end (args);

	// Print the string
	VCP_DataTx2(printbuffer,i);

	return i;
}

// Formated USB input - CR terminates input
int scanf(const char *fmt, ...)
{
	char inchar;
	va_list args;
	int i,ptr;
	char scanbuffer[120];

	//printf("\n\rFormat: %s  args=%d",fmt,args);

	ptr = 0;
	while((inchar=GetChar())!=CRR) {
        scanbuffer[ptr++] = inchar;
		//putchar(inchar);
		}
	scanbuffer[ptr] = 0;

	va_start (args, fmt);
	i = vsscanf(scanbuffer, fmt, args);
	va_end (args);

	return i;
}

/***********************************************
 *
 *  Utility Functions
 *
 ***********************************************/
void TIM15_Start(void)
{
	TIM_SetCounter(TIM15, 0);
	TIM_Cmd(TIM15,ENABLE);
}

void TIM15_Stop(void)
{
	TIM_Cmd(TIM15,DISABLE);
}

void delay_us(unsigned int ult)
{
	TIM15_Start();

	while(TIM_GetCounter(TIM15)<ult) ;
	TIM15_Stop();
}

void delay_ms(unsigned int ult)
{
	mseccountdown1 = ult;
	while(mseccountdown1) ;
}

void Toggle_Leds(int grncnt)
{
	if(!mseccountdown2) {
		mseccountdown2 = grncnt;
		if(grnflip) {
			LED1LOW;
			grnflip = 0;
			}
		else {
			LED1HIGH;
			grnflip = 1;
			}
		}
}

/***********************************************
 *
 *  Work Functions
 *
 ***********************************************/
void Short_Init(void)
{
	LED_Init();   // Leds on PC8 and PC9

	TIM3_Init();  // Setup 1msec count down timer
	TIM15_Init(); // Setup usec count down timer

	LED0OFF;

	//USART3_Init(38400);  // Setup USART3
}

void Init_RF24L01(void)
{
	//rf_mode = RF_STATE_NOTINITIALIZED;
	rf_device_class = DEVICE_CLASS;
	device_id = PICOM_DEVICE_ID;
	rf_payload_size = PAYLOAD_SIZE;
	channel_no = CHANNEL_NO;
	rf24L01_Init(RFRECEIVER);
 	//rf24L01_Init(RFSENDER);
	if(rf_state==RF_STATE_INITIALIZED_TX)  rf_mode = RF_STREAM;  // Enable RF stream mode by default
}

void PaellaM07_Init(void)
{
	LED_Init();   // Leds on PC8 and PC9

	// Configure initial port pins
	PortA_Init();
	PortB_Init();
	PortC_Init();
	PortF_Init();

	TIM3_Init();  // Setup 1msec count down timer
	TIM15_Init(); // Setup usec count down timer

	// Initialize I2C at Fast Speed to ensure correct handling of default I2C devices
	//I2C1_Init(I2C100KHZ);
	//I2C2_Init(I2C100KHZ);
	I2C1_Init(I2C400KHZ);
	//I2C2_Init(I2C400KHZ);

	SPI2_Init();

	// Pin PA8 is connected to VBUS with a pull down - if USB cable connected
	// the input is high else low
	//VBUS_Monitor_Init();

	LED0OFF;

	//USART2_Init(38400);  // Setup USART2
	//USART3_Init(38400);  // Setup USART3

	// Fire up the USB CDC subsystem
	rxtailptr = rxheadptr = 0;
	USBD_Init(&USB_Device_dev,
	            &USR_desc,
	            &USBD_CDC_cb,
	            &USR_cb);

	// See what I2C devices there are
	//Enumerate_TWI_Devices(QUIET);

	// Setup MPU9150 if enabled
	if(mpu9150state) MPU9150_Init();

	// Check for RF24L01 and initialize if present
	delay_ms(12);  // Let RF24L01 come out of power on reset

	Init_RF24L01();

#ifdef SSD1306OLED
	delay_ms(40);
	ssd1306_Init();
	ssd1306_clear(0);
	#ifdef DISPLAYSENSORDATA
		updatedisplay = ACTIVESTATE;
		updaterate = DISPLAYPDATERATE;
	#else
		updatedisplay = INACTIVE;
	#endif
#endif

#ifdef DISPLAYSENSORDATA
	memsstate = ACTIVESTATE;
#else
	memsstate = INACTIVE;  // Enable once RF24L01 is interrupt driven
#endif
}

void Printf_RCC_Clocks(void)
{
	RCC_ClocksTypeDef RCC_Clocks;
	RCC_GetClocksFreq(&RCC_Clocks);

	printf("\n\rRCC Clocks\n\r");
	printf("RCC_Clocks->SYSCLK_Frequency = %ld\n\r",RCC_Clocks.SYSCLK_Frequency);
	printf("RCC_Clocks->USBCLK_Frequency = %ld\n\r>",RCC_Clocks.USBCLK_Frequency);
}

void Signon(void)
{
	printf("\n\rPaellaM07 9DOF Wireless Sensor Processor V1.00\n\r");
	printf("For Raspberry Pi and Pi2\n\r");
#ifdef SSD1306OLED
	ssd1306_clear(0);
	ssd1306_printf(0,0,"PaellaM07 V1");
#endif

	printf("Copyright 2015 SOC Robotics, Inc.\n\r");
	Enumerate_TWI_Devices(VERBOSE);
	printf("\n\r");
	Display_RF_Status();
	printf("\n\r>");
}
void Check_RF_Connection(void)
{
	if(rf_state==RF_STATE_INITIALIZED_RX)  {
		// true if message came in
		if(RF24L01_Process_Commands()) {
			rf_inmessage = true;
			rf_cnt = mseccounter;
			}
		}

	// Check if inbound rf message occurred
	if(rf_inmessage) {
		if(mseccounter>rf_cnt+1500) {
			rf_inmessage = false;
			}
		}
}

#define RADIAN 57.296

// Work begins here
int main(void)
{
	char inchar;
	unsigned int pwmperiod;
	int i,intchar,intchar1,hours=0,minutes=0,seconds=0;
	float a,b,c,hourpos[4],minutepos[4],secondspos[4],hourangle,minuteangle,secondangle;

	// Initialize all basic functions
	//Short_Init();
	PaellaM07_Init();

	delay_ms(1000);
	Signon();

	//  Loop forever waiting on events, interrupts or USB character IO
	while(1) {

		// Check for commands coming in on the RF24L01 link and process them
		// TBD - convert to interrupt function
		Check_RF_Connection();

		// Update MEMs sensor data - keep tracking slow update sensors like BMP180 barometer
		if(memsstate==ACTIVESTATE) MEMS_Sensor_Update();

		// Update display if present -
		//if(updatedisplay==ACTIVESTATE) Update_Display();

		// Check for characters on USB CDC
		intchar = GetCharnw();
		if(intchar==-1) inchar = 0;
		else inchar = intchar;

		if(inchar) {
			if(inchar!=ESC) PutChar(inchar);

			switch(inchar) {
				case ESC:
					printf("ESC");
					PutChar(inchar=GetChar());
					GetChar();
					if(inchar=='i') printf("\n\rPaellaM07 Raspberry Pi V1.00\n\r>");
					else printf("\n\r>");
					break;
				case 's':
					Signon();
					break;
				case 'r':
					printf("\n\rRe-initialize RF24L01");
					printf("\n\r>");
					Init_RF24L01();
					break;
				case 'e':
					Enumerate_TWI_Devices(VERBOSE);
					printf("\n\r>");
					break;
				case 'c':
					//Printf_RCC_Clocks();
					break;
				case 'h':
					printf("\n\rEcho USART3 Input - hit any key to exit\n\r");
					while((intchar=GetCharnw())!='e') {
						if(intchar!=-1) PutChar3((unsigned char)intchar);
						if((intchar1=GetCharnw3())!=-1) PutChar((unsigned char) intchar1);
						}
					break;
#ifdef SSD1306OLED
				case 'l':
					printf("\n\rTest OLED");
					SSD1306_Test();
					printf("\n\r>");
					break;
#endif
				case '1':
					printf("\n\rOutput SPI data");
					while((intchar=GetCharnw())!='e') {
						//SPI2_transfer_byte(uint8_t byte);
						SPI_SendData8(SPI2, 0x55);
						delay_ms(3);
						}
					printf("\n\r>");
					break;
				case '2':
					printf("\n\rStart 4 channel PWM output");
					TIM2_Config();
					for(i=0; i<100; i++) {
						TIM2->CCR1 = (uint16_t) (((uint32_t) (TimerPeriod - 1))*ch_dutycycle[0]);
						TIM2->CCR2 = (uint16_t) (((uint32_t) (TimerPeriod - 1))*ch_dutycycle[1]);
						TIM2->CCR3 = (uint16_t) (((uint32_t) (TimerPeriod - 1))*ch_dutycycle[2]);
						TIM2->CCR4 = (uint16_t) (((uint32_t) (TimerPeriod - 1))*ch_dutycycle[3]);
						ch_dutycycle[0] += 0.01;
						ch_dutycycle[1] += 0.01;
						ch_dutycycle[2] += 0.01;
						ch_dutycycle[3] += 0.01;
						delay_ms(10);
						}
					printf("\n\r>");
					break;
				case '3':
					TIM2_Config();
					pwmperiod = 64000*0.9;
					printf("\n\rStart 4 channel PWM output - timerperiod=%d",TimerPeriod);
					while((intchar=GetCharnw())!='e') {
						for(i=2; i<2728; i++) {
							TIM2->CCR1 = pwmperiod-i;
							TIM2->CCR2 = pwmperiod-i;
							TIM2->CCR3 = pwmperiod-i;
							TIM2->CCR4 = pwmperiod-i;
							delay_us(100);
							}
						for(i=2728; i>1; i--) {
							TIM2->CCR1 = pwmperiod-i;
							TIM2->CCR2 = pwmperiod-i;
							TIM2->CCR3 = pwmperiod-i;
							TIM2->CCR4 = pwmperiod-i;
							delay_us(100);
							}
						}
					printf("\n\r>");
					break;
				case '4':
					printf("\n\rWristwatch Simulation - 'e' to exit\n\r");
					ssd1306_clear(0);
					seconds = 0;
					secondspos[0] = 29 * sin(90.0/RADIAN) + 95;
					secondspos[1] = 29 * cos(90.0/RADIAN) + 31;
					secondspos[2] = 23 * sin(90.0/RADIAN) + 95;
					secondspos[3] = 23 * cos(90.0/RADIAN) + 31;
					ssd1306_line((int)secondspos[0],(int)secondspos[1],(int)secondspos[2],(int)secondspos[3],1);
					minutes = 0;
					minutepos[0] = 22 * cos(90.0/RADIAN) + 95;
					minutepos[1] = 22 * sin(90.0/RADIAN) + 31;
					minutepos[2] = 15 * cos(90.0/RADIAN) + 95;
					minutepos[3] = 15 * sin(90.0/RADIAN) + 31;
					ssd1306_line((int)minutepos[0],(int)minutepos[1],(int)minutepos[2],(int)minutepos[3],1);
					hours = 0;
					ssd1306_circle(95, 31, 31, 0, 1);
					hourpos[0] = 13 * cos(90.0/RADIAN) + 95;
					hourpos[1] = 13 * sin(90.0/RADIAN) + 31;
					hourpos[2] = 2 * cos(90.0/RADIAN) + 95;
					hourpos[3] = 2 * sin(90.0/RADIAN) + 31;
					ssd1306_line((int)hourpos[0],(int)hourpos[1],(int)hourpos[2],(int)hourpos[3],1);
					while((intchar=GetCharnw())!='e') {
						//delay_ms(1000);
						seconds++;
						ssd1306_line((int)secondspos[0],(int)secondspos[1],(int)secondspos[2],(int)secondspos[3],0);
						if(seconds==0) secondangle = 0.0;
						else secondangle = (180.0-(6.0*seconds))/RADIAN;
						secondspos[0] = 29 * sin(secondangle) + 95;
						secondspos[1] = 29 * cos(secondangle) + 31;
						secondspos[2] = 23 * sin(secondangle) + 95;
						secondspos[3] = 23 * cos(secondangle) + 31;
						ssd1306_line((int)secondspos[0],(int)secondspos[1],(int)secondspos[2],(int)secondspos[3],1);
						printf("sp0=%d  sp1=%d  sp2=%d  sp3=%d\n\r",(int)secondspos[0],(int)secondspos[1],(int)secondspos[2],(int)secondspos[3]);
						if(seconds==60) {
							seconds = 0;
							minutes++;
							ssd1306_line((int)minutepos[0],(int)minutepos[1],(int)minutepos[2],(int)minutepos[3],0);
							if(minutes==0) minuteangle = 0.0;
							else minuteangle = (180.0-(6.0*minutes))/RADIAN;
							minutepos[0] = 22 * cos(minuteangle) + 95;
							minutepos[1] = 22 * sin(minuteangle) + 31;
							minutepos[2] = 15 * cos(minuteangle) + 95;
							minutepos[3] = 15 * sin(minuteangle) + 31;
							ssd1306_line((int)minutepos[0],(int)minutepos[1],(int)minutepos[2],(int)minutepos[3],1);
							printf("mp0=%d  mp1=%d  mp2=%d  mp3=%d\n\r",(int)minutepos[0],(int)minutepos[1],(int)minutepos[2],(int)minutepos[3]);
							if(minutes==60) {
								minutes = 0;
								hours++;
								ssd1306_line((int)hourpos[0],(int)hourpos[1],(int)hourpos[2],(int)hourpos[3],0);
								if(hours==0) hourangle = 0.0;
								else hourangle = (180.0-(30.0*hours))/RADIAN;
								hourpos[0] = 13 * cos(hourangle) + 95;
								hourpos[1] = 13 * sin(hourangle) + 31;
								hourpos[2] = 2 * cos(hourangle) + 95;
								hourpos[3] = 2 * sin(hourangle) + 31;
								ssd1306_line((int)hourpos[0],(int)hourpos[1],(int)hourpos[2],(int)hourpos[3],1);
								printf("hp0=%d  hp1=%d  hp2=%d  hp3=%d\n\r",(int)hourpos[0],(int)hourpos[1],(int)hourpos[2],(int)hourpos[3]);
								if(hours==24) hours = 0;
								}
							}
						// Draw clock face ticks
						for(i=0; i<6; i++) {

							}
						if(0) {
							}
						ssd1306_printf(8,0,"%2d:%2d:%2d",hours,minutes,seconds);
						}
					printf("\n\r>");
					break;
				case '5':
					if(hours==0) hourangle = 0.0;
					else hourangle = (360.0/hours)/RADIAN;
					hourpos[0] = 24 * cos(hourangle) + 31;
					printf("\n\hourangle=%f hourpos[0]=%f\n\r",hourangle,hourpos[0]);
					break;
				case 'x':
					printf(" flags=%x ",USART3->ICR);
					break;
				case CRR:
					printf("\n\r>");
					break;
				}
			}

		Toggle_Leds(100);
		}
}

#ifdef USE_FULL_ASSERT
/*******************************************************************************
* Function Name  : assert_failed
* Description    : Reports the name of the source file and the source line number
*                  where the assert_param error has occurred.
* Input          : - file: pointer to the source file name
*                  - line: assert_param error line source number
* Output         : None
* Return         : None
*******************************************************************************/
void assert_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
	  printf("Wrong parameters value: file %s on line %d\r\n", file, line);
  }
}
#endif


